-- CookingRef plugin by David Down
import "Turbine.Gameplay"
import "Vinny.CookingRef.CR_Data"
import "Vinny.Common.EII_ID"
import "Vinny.Common.Help"

Red,Green,Yellow = "FF2030","00FF00","FFFF00"
xpat = "<Examine:IIDDID:0x0%x+:0x700(%x+)>%[(.-)%]<\\Examine>"
xlink = "<Examine:IIDDID:0x0000000000000000:0x700%s>[%s]<\\Examine>"
local Zloc = "^(.+): .+: (%d+%.%d[NS]), (%d+%.%d[EW])$"
local Coord = "^(%d+%.%d[NnSs]),(%d+%.%d[EeWw])$"
Max_lvl = 140
List, Next = {}

function print(text) Turbine.Shell.WriteLine("<rgb=#00FFFF>CR:</rgb> "..text) end
function printh(text) print("<rgb=#00FF00>"..text.."</rgb>") end
function printc(color,text) print("<rgb=#"..color..">"..text.."</rgb>") end
function printf(fmt, ...) print(string.format(fmt, ...)) end
function printe(text) printc(Red,"Error: "..text) end

RN = {}
for id,t in pairs(CR_Data) do
	RN[t.N] = id
end
local CRv = "Cooking Ref "..Plugins["CookingRef"]:GetVersion()
CR_User = Turbine.PluginData.Load(Turbine.DataScope.Server,"CookingRef_User")
if type(CR_User) ~= "table" then
    CR_User = { [0]={} }
    printh(CRv.." data initialized.")
else printh(CRv.." data loaded.") end

player = Turbine.Gameplay.LocalPlayer.GetInstance()
pname = player:GetName()
if pname:sub(1,1)=="~" then
	printe("Session Play detected.")
	return
end

import "Vinny.CookingRef.CR_Window"
CR_window.showButton:SetEnabled( CR_User.Farm )

Plugins.CookingRef.Open = function(sender,args)
	CR_window:SetVisible( true )
	CR_window:SetZOrder( 2 )
end

plevel = player:GetLevel()
local Cook = Turbine.Gameplay.Profession.Cook
local checked = false

CR_Command = Turbine.ShellCommand()
function CR_Command:GetShortHelp() Vinny.Common.Help(help,"help") end
function CR_Command:GetHelp() Vinny.Common.Help(help) end

function nbr_item(name)
    local nr = string.match(name,"^(%d) .+")
    if nr then return tonumber(nr)
    else return 1 end
end

local function distance(dy,dx) return math.sqrt(dy*dy+dx*dx) end

local function locV(str,neg)
    local nbr = tonumber(str:sub(1,-2))
    if neg:find(str:sub(-1)) then nbr = -nbr end
    return nbr
end

function money(nr)
	local s,c = math.modf(nr/100)
	local st = s>0 and s.."s"
	if c>0 then st = (st and st.." " or "")..math.floor(c*100+0.1).."c" end
	return st
end

function hours(nr)
	local d,h = math.modf(nr/24)
	local st = d>0 and d.." days"
	if h>0 then st = (st and st..", " or "")..math.floor(h*24+0.1).." hours" end
	return st
end

-- Calculate ingredient cost/xp
function IngCost(list,tier,cost,xp,nn)
	for id,n in pairs(list) do
		local t = CR_Data[id]
		if t.T:sub(-2)=="IC" then
			cost,xp = IngCost(t.I,tier,cost,xp,n*nn)
			if t.CL==tier then xp = xp+nn*t.X end
		else
			local c = t.T:sub(-1)
			if t.T:sub(1,1)=="I" then c = t.T:sub(2,2) end
			if c=="F" then cost = cost+Farm[t.CL]*n*nn
			else cost = cost+4*t.W*n*nn end
		end
	end
	return cost,xp
end

function Check(msg)
	local CookInfo = player:GetAttributes():GetProfessionInfo(Cook)
	if not CookInfo then printe("Cooking is untrained.") return end
	CR_User[0][pname] = CookInfo:GetProficiencyLevel() +1
	local px = CR_User[pname]
	if not px then
		px = #CR_User+1
		CR_User[px] = pname
		px = tostring(px)
		CR_User[pname] = px
		print("Adding cook #"..px)
	end
	if msg then print("Checking for cooking recipes known by "..pname) end
	local n = 0
	for i = 1,CookInfo:GetRecipeCount() do
		local recipe = CookInfo:GetRecipe(i)
		local name = recipe:GetName()
		local id = RN[name] or RN[map[name]]
		local item = id and CR_Data[id]
		if not id then
			local ix = name:find(" (x",1,true) -- guild recipe?
			if ix then
				local bname = name:sub(1,ix-1)
				bname = map[bname] or bname
				id = RN[bname]
			end
		end
		if id then
			n = n+1
			if not CR_User[id] then CR_User[id] = px
			elseif not string.find(CR_User[id],px,1,true) then
				CR_User[id] = CR_User[id]..px
			end
		elseif not Prefix[name:sub(1,14)] then -- ignore alternates
			printe("Recipe not found, "..name) 
		end
	end
	checked = true
	return n
end

function CR_Command:Execute( cmd,args )
	if cmd=="cr" and (args=="help" or args=="?") then
		Vinny.Common.Help(help,args)
		return
	end
	if cmd=="cr?" then
		Vinny.Common.Help(help,"??")
		return
	end
    if args=="show" or cmd=="crw" then
        CR_window:SetVisible( true )
        return
    end
    if cmd=="crr" then
        if not CRR_window then
			import "Vinny.CookingRef.CR_Find"
		end
		CRR_window:SetVisible( true )
        return
    end
    if cmd=="crs" then
        if not CRS_window then
			import "Vinny.CookingRef.CR_Stats"
		end
        CRS_window:SetVisible( true )
        return
    end
    if cmd=="crt" then
        if #args>0 then
			List = {}
			printh("Items with a type of ("..args.."):")
			local n = 0
			for id,item in pairs(CR_Data) do
				if item.T==args then
					n = n+1
					table.insert(List,id)
					local xname = string.format(xlink,id,item.N)
					print(xname.." ("..id..")")
				end
			end
			print("Found "..n.." matches.")
			Next = next(List)
			CR_window.nextButton:SetEnabled( Next~=nil )
			return
		end
		printh("Item types found:")
		local list = { }
        for id,item in pairs(CR_Data) do
            list[item.T] = list[item.T] and list[item.T]+1 or 1
        end
		local ix = { }
        for tp in pairs(list) do
            table.insert(ix,tp)
        end
		table.sort(ix)
        for i,tp in ipairs(ix) do
            print(tp.." ("..list[tp]..")")
        end
        return
    end
    if cmd=="crf" then
		if #args<3 then print("Find what?") return end
		List = { }
		local n,str = 0,args:lower()
		printh("Cooking item names that contain '"..str.."':")
        for id,item in pairs(CR_Data) do
            local name = item.N:lower()
			if name:find(str,1,true) then
				table.insert(List,id)
				print(string.format(xlink,id,item.N))
				n = n+1
			end
        end
		print("Found "..n.." matches.")
		Next = next(List)
		CR_window.nextButton:SetEnabled( Next~=nil )
    	return
    end
    if cmd=="crn" then
		if args=="" then printe("Missing location.") return end
		local item = CR_window.item
		if item and item.B and item.T:sub(1,1)~="R" then item = CR_Data[item.B] end
		local fac = item and item.F
		if not fac then printe("No facility selected.") return end
		local reg,y,x = args:match(Zloc)
		if y then
			local inR = Rname[reg]
			local dn,y1,x1,ln = 999,locV(y,"Ss"), locV(x,"Ww")
			local y2,x2,d
			for i,t in pairs(FacLoc) do
				if inR==Region[t.Z] and t.F:find(fac,1,true)  then
					local y0,x0 = t.L:match(Coord)
					if not y then printe("Bad Loc for "..t.N) return end
					y2,x2 = locV(y0,"Ss"), locV(x0,"Ww")
					d = distance(y1-y2,x1-x2)
					if d<dn then dn = d; ln = t end
				end
			end
			local s = "The nearest "..Fac[fac].." is @ "..ln.L
			if ln.B then s = s.." in the "..ln.B end
			printf("%s at %s in %s (%.1f units away).",s,ln.N,Zone[ln.Z],dn)
		else printe("No location data in instance.") end
    	return
    end
    if cmd=="crc" then
		if args=="" then printe("Missing tier #.") return end
		local tier = tonumber(args)
		local s = tier and Tier[tier]
		if not s then printe("Invalid tier #.") return end
		if not checked then if not Check() then return end end
		local px,pt = CR_User[pname], CR_User[0][pname]
		if pt<tier then printe("No "..s.." recipes known.") return end
		printh("Crafting costs/xp for known "..s.." Cooking recipes:")
		local list,ix = { }, { }
		for id,x in pairs(CR_User) do
			local t = CR_Data[id]
			if t and t.CL==tier and t.T:sub(1,1)~="I" then
				if string.find(x,px,1,true) then
					local cost,xp = IngCost(t.I,tier,0,t.X,1)
					local s = string.format(xlink,id,t.N)
					s = string.format("%s %d/%d",s,cost,xp)
					list[s] = cost/xp
					table.insert(ix,s)
				end
			end
		end
		table.sort(ix, function (a,b) return list[a]>list[b] end )
        for i,s in ipairs(ix) do
            printf("%s = %.1f",s,list[s])
        end
    	return
    end
    if cmd=="crd" then
		if not CR_User[0][args] then printe("Not found, "..args) return end
		CR_User[0][args] = nil
		local nr = CR_User[args]
		CR_User[args] = nil
		CR_User[tonumber(nr)] = nil
		print("Removing cook #"..nr)
		local n = 0
		for id,x in pairs(CR_User) do
			local t = CR_Data[id]
			if t and string.find(x,nr) then
				n = n+1
				if x == nr then CR_User[id] = nil
				else CR_User[id] = string.gsub(x,nr,'') end
			end
		end
		print("Removed from "..n.." recipes.")
        return
    end
    if args=="check" then
        local n = Check(true)
		if n then print("Found "..n.." known recipes.") end
        return
    end
    if args=="cooks" then
		printh("Known cooks and max cooking tier:")
		for name,t in pairs(CR_User[0]) do
			print(name..' ('..Tier[t]..')')
		end
		print("To update a cook's skill record use 'check'.")
        return
    end
    if args=="tool" then
		local lvl,list = 0, {}
        for id,item in pairs(CR_Data) do
			if item.T=="TM" and item.L<=plevel then
				list[id] = item
				if item.L>lvl then lvl = item.L end
			end
        end
		print("For a level "..plevel.." cook, the best tools are:")
        for id,item in pairs(list) do
			if item.L==lvl then
				print(string.format(xlink,id,item.N).." (+"..item.c.."%)")
			end
		end
    	return
    end
    if args=="scroll" then
		if plevel<7 then printe("Level 7 required for scroll use.") return end
		local lvl,sid = 0
        for id,item in pairs(CR_Data) do
			if item.T=="SO" and item.L<=plevel then
				if item.L>lvl then
					lvl = item.L
					sid = id
				end
			end
        end
		local s = string.format(xlink,sid,CR_Data[sid].N)
		print("For a level "..plevel.." cook, the best scroll is "..s)
    	return
    end
    if args=="quest" then
		printh("Crafting quests for Cooks:")
		local lvl,sid = 0
        for i,t in ipairs(Quest) do
			printf("QL=%d %s @ %s in %s(%s): %s",t.QL,t.N,t.L,t.A,Zone[t.Z],t.Q)
			local s = ""
			for id,n in pairs(t.I) do
				if #s>0 then s=s..", " end
				if n>1 then s=s..n.."x" end
				s = s..string.format(xlink,id,CR_Data[id].N)
			end
			print("Items to cook: "..s)
        end
    	return
    end
    local id,name = args:match(xpat)
	if not id then id,name = Vinny.Common.EII_ID(args) end
	if id then
		local item = CR_Data[id]
		if item then
			Next = nil
			CR_window:SetItem( id,item )
		else printe("Unknown item, "..args) end
    	return
    end
    id = args:match("^(%x+)$")
    if id and #args==5 then
        id = string.format("%05X",tonumber(id,16))
        local item = CR_Data[id]
        if not item then printe("Undefined ID, "..id) return end
		Next = nil
        CR_window:SetItem( id,item )
        return
    end
	id = RN[args]
	if id then
		Next = nil
		CR_window:SetItem( id )
	else Vinny.Common.Help(help,args) end
end

Turbine.Shell.AddCommand( "cr;crw;crf;crs;crr;crt;crn;crc;crd;cr?",CR_Command )

Plugins.CookingRef.Unload = function(sender,args)
	Turbine.PluginData.Save(Turbine.DataScope.Server,"CookingRef_User",CR_User)
	print("Cooking Ref data saved.")
end

-- Options panel
import "Vinny.Common.Options"
Vinny.Common.Options_Init(print,CR_User,CR_window)

-- Help text
help = {
	pre = "CR",
	help = "Enter '/cr ?' for arguments, '/cr?' for commands",
	['?'] = "Unknown, enter '/cr help', '/cr?', or '/cr ?'",
	arg = {
		check = "Build a list of known recipes for displaying cook's name.",
		cooks = "List known cooks and highest tier of cooking.",
		done = "Show finished check lists.",
		quest = "Display known cooking quests.",
		scroll = "Find the best cooking scroll for your current level.",
		show = "Open Cooking Ref window.",
		tool = "Finds the best cooking tools for your current level.",
		["<link>"] = "Displays linked item in CookingRef window.",
		["<name>"] = "Use the exact name instead of a link.",
	},
	cmd = {
		crf = "Find all the cooking items containing <text>.",
		crr = "Open Recipe Find window.",
		crs = "Open Food Stats window.",
		crt = "Lists item type codes.",
		crw = "Open Cooking Ref window.",
		crd = "Delete cook data for <name>.",
	},
}
