-- CookingRef window handler
import "Turbine.UI.Lotro"
import "Vinny.Common"
import "Vinny.Common.DropMenu"
import "Vinny.Common.ToolTip"

local labelFont = Turbine.UI.Lotro.Font.Verdana14
local foreColor = Turbine.UI.Color( 0.9, 0.9, 0 )
local whiteColor = Turbine.UI.Color( 1.0, 1.0, 1.0 )
local backColor = Turbine.UI.Color( 0.0, 0.0, 0.0 )
local greyColor = Turbine.UI.Color( 0.1, 0.1, 0.1 )
local Button = Turbine.UI.Lotro.GoldButton
local Label = Turbine.UI.Label
local TextBox = Turbine.UI.TextBox
local CheckBox = Turbine.UI.Lotro.CheckBox
local Item = Turbine.UI.Lotro.ShortcutType.Item
local Alias = Turbine.UI.Lotro.ShortcutType.Alias
local Quickslot = Turbine.UI.Lotro.Quickslot
local Qsize = 36
local Guild,Raw = false,false
local Drop = "<- Drop cooking item here"
local invlist,slist

local function Shortcut(id)
	return Turbine.UI.Lotro.Shortcut(Item,"0,0x700"..id)
end

local function XP(tier,xp,rep)
	if xp>8 and not rep then xp = xp+2
	elseif xp>20 then xp = xp/35*8+12 end
	return math.floor((tier*2+xp)/4*25)
end

-- build a list of cooking ingredients in inventory
function CheckInv(pack,cnt)
	local iitem,info,id
	for i=0,cnt do
		iitem = pack:GetItem(i)
		if iitem then
			local info = iitem:GetItemInfo()
			local id = RN[info:GetName()]
			if id and CR_Data[id].T:find("I",1,true) then
				local nr = iitem:GetQuantity()
				if not invlist[id] then invlist[id] = nr
				else invlist[id] = invlist[id]+nr end
			end
		end
	end
end

-- build ingredient list, recursing if raw
local function Ingredients(tbl,list,nr)
	if not Raw and not invlist then return tbl end
	for id,n in pairs(tbl) do
		local nn,t = n*nr, CR_Data[id]
		if invlist and invlist[id] then
			local inr = invlist[id]
			if inr<nn then
				nn = nn - inr
				invlist[id] = nil
			else nn=0; invlist[id] = inr-nn end
		end
		if nn>0 then
			if t.T:sub(-2)~="IC" then
				if list[id] then list[id] = list[id] + nn
				else list[id] = nn end
			else list = Ingredients(t.I,list,nn) end
		end
	end
	return list
end

-- print an ingredient list
function PrintList(list)
	local ix = {}
	for id in pairs(list) do
		table.insert(ix,id)
	end
	table.sort(ix,function(a,b) return CR_Data[a].N < CR_Data[b].N end )
	local cost,farm = 0,0
	for i,id in pairs(ix) do
		table.insert(List,id)
		local n = list[id]
		local t = CR_Data[id]
		local tt = t.T
		local c = tt:sub(1,1)=="U" and "C" or tt:sub(-1)
		if tt:sub(1,1)=="I" then c = t.T:sub(2,2) end
		local src = Src[ c ]
		if not src then src=""; printe("Null source: "..c) end
		if t.CL then src = Tier[t.CL].." "..src end
		printf("%ix"..xlink.." (%s)",n,id,t.N,src)
		if c=="S" or c=="V" then cost = cost+4*t.W*n end
		if c=="F" then farm = farm+Farm[t.CL]*n end
	end
	if cost>0 then print("Total supplies cost is "..money(cost)) end
	if farm>0 then print("Estimated farming cost is "..money(farm)) end
	return cost,farm
end

CR_Window = class( Turbine.UI.Lotro.Window )

function CR_Window:AddField(control, text, pos, size)
	local field = control()
	field:SetParent( self )
	if text then field:SetText( text ) end
	field:SetPosition( pos.x,pos.y )
	field:SetSize( size.x,size.y )
	if control==Button or not text then return field end
	field:SetFont( labelFont )
	field:SetForeColor( text=="" and whiteColor or foreColor )
	field:SetBackColor( control==TextBox and greyColor or backColor )
	field:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft )
	return field
end

function get_nbr(field,name,min,max)
	local txt = field:GetText()
	if txt and txt~='' then
	    local val = tonumber(txt)
	    if not val or val<min or (max and val>max) then
		printe("Invalid "..name)
		return -1
	    end
	    return val
	end
end

function CR_Window:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self )
	self:SetText( "Cooking Ref." )
	self:SetSize( 340,350 )

	-- Position the window near the top center of the screen.
	local pos = CR_User.pos1 or 
		{ x=Turbine.UI.Display.GetWidth()/4*3-self:GetWidth()/4, y=self:GetHeight()/2 }
	self:SetPosition( pos.x, pos.y )

	-- Create an icon field
	self.icon = self:AddField(Quickslot, nil, {x=20,y=40}, {x=Qsize,y=Qsize} )
	self.icon:SetBackColor( Turbine.UI.Color( 0.2, 0.2, 0.2 ) )
	local Blank = self.icon:GetShortcut()
	self.icon.ShortcutChanged = function( sender, args )
		local shortcut = sender:GetShortcut()
		local itemType = shortcut:GetType()
		if itemType==0 then return end
		local itemData = shortcut:GetData()
		if sender:IsAltKeyDown() then print("Type="..itemType..", Data="..itemData) end
		if itemType~=Item then sender:SetShortcut(Blank) return end
		local Item = shortcut:GetItem()
		if not Item then printe("Item is null.") return end
		local id = itemData:sub(-5)
		if self.id==id then return end
		local item = CR_Data[id]
		if not item then
			local name = Item:GetName()
			id = RN[name] -- try finding the name
			if not id then
				printe("Unknown cooking item, "..name)
				sender:SetShortcut(Blank)
				self.name:SetText(Drop)
				return
			else item = CR_Data[id] end
		end
		Next = nil
		CR_window:SetItem( id,item )
	end

	-- Create a Name field
	self.name = self:AddField(Label, Drop, {x=60,y=40}, {x=270,y=35} )
	self.name:SetFont(Turbine.UI.Lotro.Font.TrajanPro18)

	-- Create a Type field
	self:AddField(Label, "Type:", {x=20,y=80}, {x=40,y=16} )
	self.desc = self:AddField(Label, "", {x=60,y=80}, {x=270,y=16} )

	-- Create a Worth field
	self:AddField(Label, "Worth:", {x=20,y=100}, {x=50,y=16} )
	self.worth = self:AddField(Label, "", {x=70,y=100}, {x=60,y=16} )

	-- Create a Facility field
	self:AddField(Label, "Facility:", {x=160,y=100}, {x=55,y=16} )
	self.fac = self:AddField(Label, "", {x=215,y=100}, {x=95,y=16} )

	-- Create a Find Nearest button
	self.nearButton = self:AddField(Button, "Find nearest", {x=203,y=118}, {x=110,y=20} )
	self.nearButton:SetEnabled( false )
	local slot = Quickslot()
	slot:SetParent( self.nearButton )
    slot:SetPosition( 2,2 )
    slot:SetSize( 107,15 )
    slot:SetShortcut(Turbine.UI.Lotro.Shortcut( Alias, "/crn ;loc" ))

	-- Create an XP field
	self.xp = self:AddField(Label, "", {x=20,y=120}, {x=175,y=16} )

	-- Create a Cooldown field
	self:AddField(Label, "Cooldown:", {x=20,y=140}, {x=80,y=16} )
	self.cool = self:AddField(Label, "", {x=95,y=140}, {x=115,y=16} )

	-- Create a Level field
	self:AddField(Label, "Usable at level:", {x=20,y=160}, {x=105,y=16} )
	self.level = self:AddField(Label, "", {x=127,y=160}, {x=25,y=16} )

	-- Create a Crit chance field
	self:AddField(Label, "Crit. chance:", {x=160,y=160}, {x=85,y=16} )
	self.cc = self:AddField(Label, "", {x=245,y=160}, {x=50,y=16} )

	-- Create a Quantity field
	self:AddField(Label, "Quantity made:", {x=20,y=180}, {x=120,y=16} )
	self.quan = self:AddField(Label, "", {x=125,y=180}, {x=25,y=16} )

	-- Create a Crit field
	self:AddField(Label, "Critical:", {x=160,y=180}, {x=55,y=16} )
	self.crit = self:AddField(Label, "", {x=215,y=180}, {x=20,y=16} )

	-- Create an Used-in button
	self.usedButton = self:AddField(Button, "Used in", {x=40,y=205}, {x=95,y=20} )
	self.usedButton:SetEnabled( false )
	self.usedButton.Click = function( sender,args )
		printh(self.item.N.." is used in:")
		List = {}
		local n = 0
		local iid,lid = self.id
		local px = CR_User[pname]
		for id,item in pairs(CR_Data) do
			local tr = Tier[item.CL or item.B and CR_Data[item.B].CL] or ""
			local inr = item.I and item.I[iid]
			if inr and (Guild or not item.T:find("G",1,true)) then
				n = n+1
				lid = id
				table.insert(List,id)
				local rk = CR_User[id]
				if px and not (rk and string.find(rk,px,1,true)) then
					tr = "<rgb=#FF2030>"..tr.."</rgb>"
				end
				printf(xlink.." (%s) %s",id,item.N,inr,tr)
			end
			local inr = Guild and item.G and item.G.I and item.G.I[iid]
			if inr then
				n = n+1
				if id~=lid then table.insert(List,id) end
				printf(xlink.." (%s) %s",id,item.G.N,inr,tr)
			elseif Guild and item.G and not item.G.I then
				printe("No I for "..item.G.N.."("..item.N..")" )
			end
		end
		print("Found "..n.." uses.")
		Next = next(List)
		self.nextButton:SetEnabled( Next~=nil )
		self.farmButton:SetEnabled( false )
	end

	-- Guild check box
	local box = self:AddField(CheckBox, " Guild", {x=140,y=207}, {x=65,y=16} )
	box.CheckedChanged = function( sender,args )
		Guild = sender:IsChecked()
		print((Guild and "Enabled " or "Disabled ").."Guild recipes.")
	end
	Vinny.Common.ToolTip(box,2,-20,"Include Guild recipes.",240)

	-- Create a Critical button
	self.critButton = self:AddField(Button, "Critical", {x=210,y=205}, {x=95,y=20} )
	self.critButton:SetEnabled( false )
	self.critButton.Click = function( sender,args )
		self:SetItem( self.item.C )
	end

	-- Create an ingredient button
	self.ingrButton = self:AddField(Button, "Ingredients", {x=40,y=230}, {x=95,y=20} )
	self.ingrButton:SetEnabled( false )
	self.ingrButton.Click = function( sender,args )
		local s = Raw and "Raw i" or "I"
		printh(s.."ngredients for "..self.item.N..":")
		slist = Ingredients(self.item.I,{},1)
		List = {}
		local v,f = PrintList(slist)
		if Raw and v>0 and f>0 then
			local q = self.item.Q or 1
			print("Total cost per item is "..money((v+f)/q))
		end
		Next = next(List)
		self.nextButton:SetEnabled( Next~=nil )
		self.farmButton:SetEnabled( Raw )
	end

	-- Raw check box
	local box = self:AddField(CheckBox, " Raw", {x=140,y=232}, {x=55,y=16} )
	box.CheckedChanged = function( sender,args )
		Raw = sender:IsChecked()
		print((Raw and "Enabled " or "Disabled ").."Raw ingredients.")
	end
	Vinny.Common.ToolTip(box,2,-20,"Convert crafted ingredients to raw.",240)

	-- Create a Guild button
	self.guildButton = self:AddField(Button, "Guild", {x=210,y=230}, {x=95,y=20} )
	self.guildButton:SetEnabled( false )
	self.guildButton.Click = function( sender,args )
		local gitem = self.item.G
		gitem.B = self.id
		self.name:SetText( gitem.N )
		if gitem.X then
			local xp = self.xp:GetText()
			local i = xp:find(":",1,true)+1
			self.xp:SetText(xp:sub(1,i)..gitem.X)
		end
		self.cool:SetText( hours(gitem.H) )
		self.quan:SetText( gitem.Q or 1 )
		self.crit:SetText( "" )
		self.item = gitem
		self.ingrButton:SetEnabled( true )
		self.guildButton:SetEnabled( false )
		self.baseButton:SetEnabled( true )
		self.recButton:SetEnabled( true )
	end

	-- Create a Base button
	self.baseButton = self:AddField(Button, "Base Recipe", {x=40,y=255}, {x=95,y=20} )
	self.baseButton:SetEnabled( false )
	self.baseButton.Click = function( sender,args )
		self:SetItem( self.item.B )
	end

	-- Create a Recipe button
	self.recButton = self:AddField(Button, "Recipe", {x=140,y=255}, {x=85,y=20} )
	self.recButton:SetEnabled( false )
	self.recButton.Click = function( sender,args )
		self:SetItem( self.item.R )
	end

	-- Create a Next button
	self.nextButton = self:AddField(Button, "Next", {x=230,y=255}, {x=75,y=20} )
	self.nextButton:SetEnabled( false )
	self.nextButton.Click = function( sender,args )
		local id = List[Next]
		Next = next(List,Next)
		self:SetItem( id )
	end

	-- Create a Make field
	self:AddField(Label, "# to make:", {x=55,y=287}, {x=95,y=16} )
	self.make = self:AddField(TextBox, "", {x=135,y=287}, {x=27,y=16} )

	-- Create a Shopping-list button
	self.shopButton = self:AddField(Button,"Shopping-list",{x=175,y=285},{x=115,y=20})
	self.shopButton:SetEnabled( false )
	self.shopButton.Click = function( sender,args )
		local nr = get_nbr(self.make,"# to make",1,100)
		if nr<1 then return end
		invlist = {}
		local vault = player:GetVault()
		if not vault:IsAvailable() then
			printc(Yellow,"Warning: Vault contents not accessable.")
			vault = nil
		else CheckInv(vault,vault:GetCapacity()) end
		local pack = player:GetBackpack()
		CheckInv(pack,pack:GetSize())
		local q = self.item.Q
		local inr = q and math.ceil(nr/q) or nr
		slist = Ingredients(self.item.I,{},inr)
		if not next(slist) then
			printh("No additional ingredients are needed.")
			return
		end
		printh("To make "..nr.." "..self.item.N..", you also need:")
		PrintList(slist)
		invlist = nil
		self.farmButton:SetEnabled( true )
	end

	-- Create a Save Farmer List button
	self.farmButton = self:AddField(Button, "Save Farmer List", {x=40,y=310}, {x=125,y=20} )
	self.farmButton:SetEnabled( false )
	self.farmButton.Click = function( sender,args )
		local farm = {}
		for id,n in pairs(slist) do
			if CR_Data[id].T:sub(1,2)=="IF" then farm[id] = n end
		end
		if not next(farm) then
			printe("No farming ingredients needed.")
			return
		end
		CR_User.Farm = farm
		print("List saved.")
		self.showButton:SetEnabled( true )
	end

	-- Create a Show List button
	self.showButton = self:AddField(Button, "Show Farmer List", {x=170,y=310}, {x=130,y=20} )
	self.showButton.Click = function( sender,args )
		printh("Farming ingredients:")
		PrintList(CR_User.Farm)
	end

end

function CR_Window:SetItem(id,item,xname)
    if not item then item = CR_Data[id] end
    if not xname then xname = string.format(xlink,id,item.N) end
	local s,px = "", CR_User[item.B or id]
	if px then
		for i = 1,#px do
			pn = CR_User[tonumber(string.sub(px,i,i))]
			if #s>0 then s = s..", " end
			s = s..pn
		end
		s = " ("..s..")"
	end
    print(xname..s)
	self.id = id
	self.item = item
	local bitem = item
	local c1,c2,c3 = item.T:sub(1,1), item.T:sub(2,2), item.T:sub(3,3)
	if item.B and (c1=="U" or c1=="T") and not item.X then bitem = CR_Data[item.B] end
	self.icon:SetShortcut( Shortcut(id) )
	self.name:SetText( item.N )
	local tp = Ctype[c1]
	local tier = bitem and Tier[bitem.CL]
	if c1=="R" then
		local bitem = CR_Data[item.B]
		if bitem and bitem.B then bitem = CR_Data[bitem.B] end
		local tr = bitem and Tier[bitem.CL].." " or ""
		local t3 = #c3>0 and ", "..Rtype[c3] or ""
		tp = tr..tp.." ("..Rtype[c2]..t3..")"
	elseif c1=="I" or c1=="S" or c1=="T" then
		if c3=="f" then c2 = c3 end
		local s2 = c3=="H" and ", "..Src.H or ""
		tp = tp.." ("..(tier and tier.." " or "")..Src[c2]..s2..")"
	elseif c1=="U" then
		tp = tp.." ("..Utype[c2]..")"
	elseif c1=="G" then
		tp = tp.." ("..Gtype[c2]..")"
	elseif c1=="O" then tp = tier.." "..tp end
	if c3=="I" then tp = tp..", Ingredient" end
	self.desc:SetText(tp)
	self.worth:SetText( item.W and money(item.W) )
	self.fac:SetText( Fac[bitem.F or "N"] )
	self.nearButton:SetEnabled( bitem.F~=nil )
	local xp,n = bitem.X
	if xp then n = XP(bitem.CL,xp,item.T:sub(1,2)=="Gr") end
	self.xp:SetText(xp and tier.." XP: "..xp.."  (+"..n..")")
	self.cool:SetText( item.H and hours(item.H) )
	self.level:SetText(bitem.L)
	self.cc:SetText(item.c and "+"..item.c.."%")
	self.quan:SetText(item.Q or bitem.X and 1)
	self.crit:SetText( type(item.C)=="number" and item.C )
	self:SetVisible( true )
	self.usedButton:SetEnabled( item.T:find("I",1,true) )
	self.critButton:SetEnabled( type(item.C)=="string" )
	self.ingrButton:SetEnabled( item.I~=nil )
	self.guildButton:SetEnabled( item.G~=nil )
	self.baseButton:SetEnabled( item.B~=nil )
	self.recButton:SetEnabled( item.R~=nil )
	self.nextButton:SetEnabled( Next~=nil )
	self.shopButton:SetEnabled( item.I~=nil )
	self.make:SetText("1")
end

CR_window = CR_Window()

-- Set Escape action
CR_window:SetWantsKeyEvents( true )
CR_window.KeyDown = function(sender, args)
	if( args.Action == Turbine.UI.Lotro.Action.Escape ) then
		CR_window:SetVisible( false )
		if CRR_window then CRR_window:SetVisible( false ) end
		if CRS_window then CRS_window:SetVisible( false ) end
	end
end
